@@ -1,9 +1,6 @@
-from pythonql.algebra.operator import plan_from_list
-from pythonql.algebra.operators import *
 from pythonql.PQTuple import PQTuple
+from pythonql.PQTable import PQTable
 from pythonql.helpers import flatten
-from pythonql.Rewriter import rewrite
-from pythonql.debug import Debug
 import json
 import types
 
@@ -119,16 +116,8 @@ def emptyTuple(schema):
 def PyQuery( clauses, prior_locs, returnType ):
   data = []
   data.append( emptyTuple([]) )
-
-  clauses = list(clauses)
-  clauses.reverse()
-
-  plan = plan_from_list(clauses)
-  plan = rewrite(plan, prior_locs)
-
-  if Debug().print_optimized:
-      print("Rewritten query:",plan)
-  data = plan.execute(data, prior_locs)
+  for c in clauses:
+    data = processClause(c, data, prior_locs)
   if returnType == "gen":
     return data
   elif returnType == "list":
@@ -138,21 +127,44 @@ def PyQuery( clauses, prior_locs, returnType ):
   else:
     return dict(data)
 
+# Process clauses
+def processClause(c, table, prior_locs):
+  if c["name"] == "select":
+    return processSelectClause(c, table, prior_locs)
+  elif c["name"] == "for":
+    return processForClause(c, table, prior_locs)
+  elif c["name"] == "let":
+    return processLetClause(c, table, prior_locs)
+  elif c["name"] == "match":
+    return processMatchClause(c, table, prior_locs)
+  elif c["name"] == "count":
+    return processCountClause(c, table, prior_locs)
+  elif c["name"] == "where":
+    return processWhereClause(c, table, prior_locs)
+  elif c["name"] == "groupby":
+    return processGroupByClause(c, table, prior_locs)
+  elif c["name"] == "orderby":
+    return processOrderByClause(c, table, prior_locs)
+  elif c["name"] == "window":
+    return processWindowClause(c, table, prior_locs)
+  else:
+    raise Exception("Unknown clause %s encountered" % c[0] )
+  
 # Process Select clause
 # We still keep that feature of generating tuples for now
 def processSelectClause(c, table, prior_lcs):
   # If this is a list/set comprehension:
-  if c.expr:
+  if "expr" in c:
     # Compile the expression:
-    e = compile(c.expr.lstrip(), '<string>','eval')
+    e = compile(c["expr"].lstrip(), '<string>','eval')
     for t in table:
       lcs = prior_lcs
       lcs.update(t.getDict())
       yield eval(e,globals(),lcs)
 
   else:
-    k_expr = compile(c.key_expr.lstrip(),'<string>','eval')
-    v_expr = compile(c.value_expr.lstrip(),'<string>','eval')
+    k_expr = compile(c["key"].lstrip(),'<string>','eval')
+    v_expr = compile(c["value"].lstrip(),'<string>','eval')
     for t in table:
       lcs = prior_lcs
       lcs.update(t.getDict())
@@ -164,18 +176,18 @@ def processSelectClause(c, table, prior_lcs):
 # product of the input table with new sequence
 def processForClause(c, table, prior_lcs):
   new_schema = None
-  comp_expr = compile(c.expr.lstrip(), "<string>", "eval")
+  comp_expr = compile(c["expr"].lstrip(), "<string>", "eval")
 
   for t in table:
     if not new_schema:
       new_schema = dict(t.schema)
-      for (i,v) in enumerate(c.vars):
+      for (i,v) in enumerate(c["vars"]):
         new_schema[v] = len(t.schema) + i
 
     lcs = prior_lcs
     lcs.update(t.getDict())
     vals = eval(comp_expr, globals(), lcs)
-    if len(c.vars) == 1:
+    if len(c["vars"]) == 1:
       for v in vals:
         new_t_data = list(t.tuple)
         new_t_data.append(v)
@@ -185,7 +197,7 @@ def processForClause(c, table, prior_lcs):
     else:
       for v in vals:
         unpack_expr = "[ %s for %s in [ __v ]]" % (
-                      '(' + ",".join(c.vars) + ')', c.unpack)
+                      '(' + ",".join(c["vars"]) + ')', c["unpack"])
         unpacked_vals = eval(unpack_expr, globals(), {'__v':v})
         new_t_data = list(t.tuple)
         for tv in unpacked_vals[0]:
@@ -196,26 +208,26 @@ def processForClause(c, table, prior_lcs):
 # Process the let clause. Here we just add a variable to each
 # input tuple
 def processLetClause(c, table, prior_lcs):
-  comp_expr = compile(c.expr.lstrip(), "<string>", "eval")
+  comp_expr = compile(c["expr"].lstrip(), "<string>", "eval")
   new_schema = None
   for t in table:
 
     if not new_schema:
       new_schema = dict(t.schema)
-      for (i,v) in enumerate(c.vars):
+      for (i,v) in enumerate(c["vars"]):
         new_schema[v] = len(t.schema) + i
 
     lcs = prior_lcs
     lcs.update(t.getDict())
     v = eval(comp_expr, globals(), lcs)
-    if len(c.vars) == 1:
+    if len(c["vars"]) == 1:
       t.tuple.append(v)
       new_t = PQTuple( t.tuple, new_schema )
       yield new_t
 
     else:
       unpack_expr = "[ %s for %s in [ __v ]]" % (
-                      '(' + ",".join(c.vars) + ')', c.unpack) 
+                      '(' + ",".join(c["vars"]) + ')', c["unpack"]) 
       unpacked_vals = eval(unpack_expr, globals(), {'__v':v})
       new_t_data = list(t.tuple)
       for tv in unpacked_vals[0]:
@@ -223,103 +235,14 @@ def processLetClause(c, table, prior_lcs):
       new_t = PQTuple(new_t_data, new_schema)
       yield new_t
 
-# Process a join
-def processJoin(c, table, prior_lcs, left_arg, right_arg):
-  new_schema = None
-  left_conds = c.left_conds
-  right_conds = c.right_conds
- 
-  join_type = 'nl'
-  dir = 'right'
-  if c.hint:
-    join_type = c.hint['join_type']
-    dir = c.hint['dir']
-
-  if dir == 'left':
-    left_arg,right_arg = right_arg,left_arg
-
-  r_init_data = []
-  r_init_data.append( emptyTuple([]) )
-
-  # Build an index on the right relation, if we're doing
-  # an index join. 
-  index = None
-  if join_type == 'index':
-    index = {}
-    r_data = r_init_data
-    r_data = right_arg.execute(r_data, prior_lcs)
-
-    for t in r_data:
-        index_tuple = [] 
-        for rcond in right_conds:
-            lcs = prior_lcs
-            lcs.update(t.getDict())
-            rcond_val = eval(rcond, globals(), lcs)
-            index_tuple.append( rcond_val )
-        index_tuple = tuple(index_tuple)
-        if not index_tuple in index:
-            index[ index_tuple ] = []
-        index[ index_tuple ].append( t )
-
-  # Iterate over the tuples of the left relation and
-  # compute the tuple of condition vars
-
-  table = left_arg.execute(table, prior_lcs)
-  for t in table:
-     cond_tuple = []
-     for lcond in left_conds:
-         lcs = prior_lcs
-         lcs.update(t.getDict())
-         lcond_val = eval(lcond, globals(), lcs)
-         cond_tuple.append( lcond_val )
-     cond_tuple = tuple(cond_tuple)
-
-     if index:
-         if cond_tuple in index:
-             for t2 in index[cond_tuple]:
-                 if not new_schema:
-                     new_schema = dict(t.schema)
-                     for i,_ in enumerate(t2):
-                         v = [x for x in t2.schema.items() if x[1]==i][0][0]
-                         new_schema[v] = len(new_schema) + i
-                 new_t_data = list(t.tuple)
-                 new_t_data += list(t2.tuple)
-                 new_t = PQTuple(new_t_data, new_schema)
-                 yield new_t
-         else:
-             continue
-     else:
-          r_data = r_init_data
-          r_data = right_arg.execute(r_data, prior_lcs)
-
-          for t2 in r_data:
-              rcond_tuple = []
-              for rcond in right_conds:
-                  lcs = prior_lcs
-                  lcs.update(t2.getDict())
-                  rcond_val = eval(rcond, globals(), lcs)
-                  rcond_tuple.append( rcond_val )
-
-              rcond_tuple = tuple(rcond_tuple)
-              if cond_tuple == rcond_tuple:
-                 if not new_schema:
-                     new_schema = dict(t.schema)
-                     for i,_ in enumerate(t2):
-                         v = [x for x in t2.schema.items() if x[1]==i][0][0]
-                         new_schema[v] = len(new_schema) + i
-                 new_t_data = list(t.tuple)
-                 new_t_data += list(t2.tuple)
-                 new_t = PQTuple(new_t_data, new_schema)
-                 yield new_t
-
 # Process the match claise
 def processMatchClause(c, table, prior_lcs):
-  clause_expr = compile(c.expr, "<string>", "eval")
+  clause_expr = compile(c['expr'], "<string>", "eval")
 
   # Fetch and compile all expressions in the
   # pattern match clause
   e_patterns = []
-  patterns = list(c.pattern)
+  patterns = list(c['pattern'])
   while patterns:
     p = patterns.pop() 
     if 'expr_cond' in p:
@@ -334,7 +257,7 @@ def processMatchClause(c, table, prior_lcs):
   for t in table:
     if not new_schema:
       new_schema = dict(t.schema)
-      for (i,v) in enumerate(c.vars):
+      for (i,v) in enumerate(c["vars"]):
         new_schema[v] = len(t.schema) + i
 
     lcs = prior_lcs
@@ -345,10 +268,10 @@ def processMatchClause(c, table, prior_lcs):
       if not hasattr(v, '__contains__'):
         continue
       
-      new_t_data = list(t.tuple) + [None]*len(c.vars)
+      new_t_data = list(t.tuple) + [None]*len(c['vars'])
       new_t = PQTuple(new_t_data, new_schema)
 
-      if match_pattern(c.pattern, c.exact, v, new_t, lcs):
+      if match_pattern(c['pattern'], c['exact'], v, new_t, lcs):
         yield new_t
 
 def match_pattern(ps, isExact, v, new_t, lcs):
@@ -394,7 +317,7 @@ def processCountClause(c, table, prior_lcs):
 
     if not new_schema:
       new_schema = dict(t.schema)
-      new_schema[c.var] = len(t.schema)
+      new_schema[c["var"]] = len(t.schema)
 
     new_t = PQTuple( t.tuple + [i], new_schema )
     yield new_t
@@ -402,9 +325,9 @@ def processCountClause(c, table, prior_lcs):
 # Process the group-by
 def processGroupByClause(c, table, prior_lcs):
   gby_aliases = [g if isinstance(g,str) else g[1]
-                                for g in c.groupby_list]
+                                for g in c["groupby_list"]]
   gby_exprs = [g if isinstance(g,str) else g[0]
-                                for g in c.groupby_list]
+                                for g in c["groupby_list"]]
   comp_exprs = [compile(e,'<string>','eval') for e in gby_exprs]
   grp_table = {}
 
@@ -452,7 +375,7 @@ def processGroupByClause(c, table, prior_lcs):
 
 # Process where clause
 def processWhereClause(c, table, prior_lcs):
-  comp_expr = compile(c.expr.lstrip(),"<string>","eval")
+  comp_expr = compile(c["expr"].lstrip(),"<string>","eval")
   for t in table:
     lcs = prior_lcs
     lcs.update(t.getDict())
@@ -466,8 +389,8 @@ def processOrderByClause(c, table, prior_lcs):
   # For each sort we first need to compute a sort value (could
   # be some expression)
 
-  sort_exprs = [ compile(os[0].lstrip(),"<string>","eval") for os in c.orderby_list]
-  sort_rev = [ o[1]=='desc' for o in c.orderby_list]
+  sort_exprs = [ compile(os[0].lstrip(),"<string>","eval") for os in c["orderby_list"]]
+  sort_rev = [ o[1]=='desc' for o in c["orderby_list"]]
 
   def computeSortSpec(tup,sort_spec):
     lcs = prior_lcs
@@ -530,20 +453,20 @@ def check_start_condition(all_vars,clause,locals,var_mapping):
   locals.update( start_bindings )
 
   #evaluate the when condition
-  return eval( clause.s_when, globals(), locals )
+  return eval( clause["s_when"], globals(), locals )
   
 # Check the end condition of the window.
 
 def check_end_condition(vars,clause,locals,var_mapping):
   # If there is no 'when' clause, return False
-  if not clause.e_when:
+  if not clause["e_when"]:
     return False
 
   end_vars = set(vars.keys()).intersection( set(var_mapping.keys()))
   end_binding = { var_mapping[v] : vars[v] for v in end_vars }
 
   locals.update( end_binding )
-  res = eval( clause.e_when, globals(), locals)
+  res = eval( clause["e_when"], globals(), locals)
 
   return res
 
@@ -554,21 +477,21 @@ def processWindowClause(c, table, prior_lcs):
 
   # Create window variable name mapping
   var_mapping = {}
-  for v in c.vars:
-    var_mapping[v] = c.vars[v]
+  for v in c["vars"]:
+    var_mapping[v] = c["vars"][v]
 		
   for t in table:
     if not schema:
       schema = t.schema
       # Create a new schema with window variables added
       new_schema = dict(t.schema)
-      for v in c.vars:
-        new_schema[c.vars[v]] = len(new_schema)
+      for v in c["vars"]:
+        new_schema[c["vars"][v]] = len(new_schema)
 
     lcs = dict(prior_lcs)
     lcs.update(t.getDict())
     # Evaluate the binding sequence
-    binding_seq = list(eval(c.binding_seq, globals(), lcs))
+    binding_seq = list(eval(c["in"], globals(), lcs))
 
     # Create initial window variables
 
@@ -581,7 +504,7 @@ def processWindowClause(c, table, prior_lcs):
       # Try to open a new window
       # in case of tumbling windows, only open a
       # window if there are no open windows
-      if not c.tumbling or (c.tumbling and not open_windows):
+      if not c["tumbling"] or (c["tumbling"] and not open_windows):
         vars = make_window_vars()
         fill_in_start_vars(vars,binding_seq,i)
         if check_start_condition(vars,c,dict(lcs),var_mapping):
@@ -603,7 +526,7 @@ def processWindowClause(c, table, prior_lcs):
           
     #close or remove all remaining open windows
     #if only is specified, we ignore non-closed windows
-    if not c.only:
+    if not c["only"]:
       closed_windows.extend(open_windows)
     
     # create a new tuple by extending the tuple from previous clauses
